%{

#include "headers.h"
#include "cfgvar.h"
#include "servers.h"
#include "rsh.h"
#include "opt.h"
#include "storage.h"

int yylex();
int yyerror();

extern int ipcacfglineno;

%}

/*
 * Token value definition
 */
%union {
	int tv_int;
	char *tv_char;
	struct in_addr tv_ip;
}

/*
 * Token types returned by scanner
 */
%token	<tv_char>	TOK_STRING

/*
 * Reserved words
 */
%token				ERROR
%token				EQ
%token				AT
%token				SEMICOLON
%token				SLASH
%token				IFACE
%token				INONLY
%token				PROMISC
%token				FILTER

%token				MEMSIZE
%token				RSH
%token				DUMP
%token				CHROOT
%token				UID
%token				GID
%token				AGGR
%token				STRIP
%token				DENY
%token				ALLOW
%token				TIMEOUT
%token				TTL
%token				PIDFILE

%token				ADMIN
%token				BACKUP
%token				DEFAULT
%token				VIEW_ONLY

%type	<tv_int>	privlevel
%type	<tv_int>	IFlags
%type	<tv_char>	IFilter
%type	<tv_ip>		at_ip

%%
cfg_file:
	| sequence;

sequence: block
	| sequence block
	;

ps:
	| SEMICOLON;

privlevel:
	{ $$ = 2; }
	| ADMIN	{ $$ = 5; }
	| BACKUP	{ $$ = 3; }
	| DEFAULT	{ $$ = 2; }
	| VIEW_ONLY { $$ = 1; }
	| DENY { $$ = 0; }
	;

IFlags:
	{ $$ = IFLAG_NONE; }
	| INONLY { $$ = IFLAG_INONLY; }
	| PROMISC { $$ = IFLAG_PROMISC; }
	| IFlags INONLY { $$ |= IFLAG_INONLY; }
	| IFlags PROMISC { $$ |= IFLAG_PROMISC; }
	;

IFilter:
	{ $$ = NULL; }
	| FILTER TOK_STRING { $$ = $2; }
	;

block:
	PIDFILE EQ TOK_STRING ps {
		if(pidfile)
			return yyerror("Pidfile already defined");
		pidfile = $3;
	}
	| TTL EQ TOK_STRING ps {
		default_ttl = atoi($3);
		free($3);
	}
	| IFACE TOK_STRING IFlags IFilter ps {
		cfg_add_iface($2, $3, $4);
		free($2);
	}
	| DUMP EQ TOK_STRING ps {
		if( conf->dump_table_file )
			free(conf->dump_table_file);
		conf->dump_table_file = $3;
	}
	| RSH ALLOW at_ip ps {
		if(add_server(rsh_server, "RSH Server", &($3)))
			return yyerror("Failed to install RSH server");
		fprintf(stderr, "Configured RSH Server listening at %s\n",
			inet_ntoa($3));
	}
	| RSH DENY at_ip ps {
		fprintf(stderr, "Warning: Option at line %d has no effect\n",
			ipcacfglineno);
	}
	| RSH TIMEOUT EQ TOK_STRING ps {
		int to_ms;
		to_ms = atoi($4);
		free($4);
		if(to_ms < 0)
			to_ms = -1;	/* INFTIM */
		else
			to_ms = to_ms * 1000;
		rsh_rw_timeout = to_ms;
	}
	| RSH TOK_STRING privlevel ps {
		cfg_add_rsh_host("", $2, $3);
		free($2);
	}
	| RSH AT TOK_STRING privlevel ps {
		cfg_add_rsh_host("", $3, $4);
		free($3);
	}
	| RSH TOK_STRING AT TOK_STRING privlevel ps {
		cfg_add_rsh_host($2, $4, $5);
		free($2); free($4);
	}
	| CHROOT EQ TOK_STRING ps {
		if(conf->chroot_to)
			free(conf->chroot_to);
		conf->chroot_to = $3;
	}
	| UID EQ TOK_STRING ps {
		int id = $3 ? atoi($3) : -1;
		free($3);
		if(id < 0 || id > 65535)
			return yyerror("Inappropriate UID value");
		conf->set_uid = id;
	}
	| GID EQ TOK_STRING ps {
		int id = $3 ? atoi($3) : -1;
		free($3);
		if(id < 0 || id > 65535)
			return yyerror("Inappropriate UID value");
		conf->set_gid = id;
	}

	| MEMSIZE EQ TOK_STRING ps {
		char *p = $3;

		conf->memsize = atoi($3);
		if(conf->memsize <= 0) {
			free($3);
			return yyerror("Invalid memory limit");
		}
		while(*++p) {
			if(*p >= '0' && *p <= '9')
				continue;
			switch(*p) {
			case ' ':
				break;
			case 'k':
				conf->memsize *= 1024;
				break;
			case 'm':
				conf->memsize *= 1024 * 1024;
				break;
			case 'g':
				conf->memsize *= 1024 * 1024 * 1024;
				break;
			case 'e':
				conf->memsize *= sizeof(flow_el_t);
				break;
			default:
				fprintf(stderr,
					"Invalid memory limit at line %d "
					"near '%c': %s\n",
					*p, ipcacfglineno, $3);
				return -1;
			}
		}
		free($3);
	}
	| AGGR TOK_STRING SLASH TOK_STRING STRIP TOK_STRING ps {
		if(cfg_add_atable($2, $4, $6) != 0) {
			free($2);
			free($4);
			return yyerror("Parse error");
		};
		free($2);
		free($4);
	}
	;

at_ip:
	{ $$.s_addr = INADDR_ANY; }
	| AT TOK_STRING {
		if(inet_aton($2, &($$)) != 1)
			return yyerror("IP address expected");
	}
	;

%%

int
yyerror(char *s) {
	fprintf(stderr, "Config parse error near line %d: %s\n", ipcacfglineno, s);
	return -1;
};

