/*-
 * Copyright (c) 2001 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: disp.c,v 1.18 2002/12/25 10:09:47 vlm Exp $
 */

#include "ipcad.h"
#include "cfgvar.h"
#include "storage.h"
#include "opt.h"

void show_stats(FILE *f);

#define ROOM_FOR_IPADDR	17
#define	C(f)	(unsigned char)((f) & 0xff)

void
print_ip(FILE *f, struct in_addr ip) {
	fprintf(f, "%d.%d.%d.%d",
		C(ip.s_addr),
		C(ip.s_addr >> 8),
		C(ip.s_addr >> 16),
		C(ip.s_addr >> 24)
	);
}

void
print_aligned_ip(FILE *f, struct in_addr ip) {
	int z;

	z = fprintf(f, " %d.%d.%d.%d",
		C(ip.s_addr),
		C(ip.s_addr >> 8),
		C(ip.s_addr >> 16),
		C(ip.s_addr >> 24)
	);

	while(z++ < ROOM_FOR_IPADDR)
		putc(' ', f);
}


int
display(FILE *f, int ischeckpoint) {
	flow_t *flows;
	flow_t *flow;
	int entries;
	time_t storage_created;
	time_t stalled;
	long long ex_packets;
	long long ex_bytes;
	flow_storage_t *storage;
	int ret;

	storage = ischeckpoint ? &checkpoint_storage : &main_storage;

	lock_storage(storage);
		stalled = storage->first_miss;
		ex_packets = storage->missed_packets;
		ex_bytes = storage->missed_bytes;
		storage_created = storage->create_time;

		ret = get_flow_table(storage, &flows, &entries);
	unlock_storage(storage);

	if(ret == -1) {
		fprintf(f, "Memory allocation error.\n");
		return -1;
	}

	fprintf(f, "\n");
	fprintf(f, "   Source           Destination              Packets               Bytes\n");

	for(flow = flows; entries > 0; entries--, flow++) {
		print_aligned_ip(f, flow->src);
		print_aligned_ip(f, flow->dst);
		fprintf(f, "%18u %19qu\n",
			flow->packets, flow->bytes);
	}

	fprintf(f, "\n");

	if(flows) free(flows);


	if(storage_created) {
		time_t age = (time(NULL) - storage_created) / 60;
		fprintf(f, "Accounting data age is %5ld\n", (long)age);
		fprintf(f, "Accounting data age exact %ld\n",
			time(NULL) - storage_created);
	}

	if(ex_packets)
		fprintf(f,
			"Accounting threshold exceeded for %llu packets "
			"and %llu bytes\n", ex_packets, ex_bytes);

	if(stalled)
		fprintf(f, "Information incomplete since %s", ctime(&stalled));

	if(!ischeckpoint)
		show_stats(f);

	return 0;
}

void display_capture_stats(FILE *);

void
show_stats(FILE *f) {
	size_t used_memory = main_storage.entries *
		sizeof(*main_storage.buckets[0]);

	display_capture_stats(f);

	fprintf(f, "Entries made: %d\n", main_storage.entries);

	if(conf->memsize)   
		fprintf(f, "Memory usage: %d%% (%u from %u)\n",
			used_memory * 100 / conf->memsize,
			used_memory, conf->memsize);
	else
		fprintf(f, "Memory usage: %u kbytes.\n", used_memory >> 10);

	fprintf(f, "Free slots for clients: %d\n", max_clients);

	ipcad_uptime(f);
	system_uptime(f);
  
};

int
display_internal_averages(FILE *f, const char *ifname) {
	packet_source_t *ps;

	/* Find the interface */
	for(ps = conf->packet_sources_head; ps; ps = ps->next) {
		if(strcmp(ps->ifname, ifname) == 0)
			break;
	}

	if(!ps)
		return -1;

	fprintf(f, "  %.0f minute average rate %lu bits/sec, %lu packets/sec\n",  
		ps->avg_period / 60,
		ps->bps_lp * 8,
		ps->pps_lp 
	);

	return 0;
}

void
show_version(FILE *f) {
	fprintf(f,
		IPCAD_VERSION "\n"
		IPCAD_COPYRIGHT "\n"
		"\n"
	);

	ipcad_uptime(f);
	system_uptime(f);
}


void
ipcad_uptime(FILE *f) {
	fprintf(f, "IPCAD uptime is");
	display_uptime(f, time(NULL) - self_started.tv_sec);
}


void
display_uptime(FILE *f, time_t uptime) {
	int days, hrs, mins, secs;

	days = uptime / 86400;
	uptime %= 86400;
	hrs = uptime / 3600;
	uptime %= 3600;
	mins = uptime / 60;
	secs = uptime % 60;

	if(days > 0)
		fprintf(f, " %d day%s", days, days>1?"s":"");
	if(hrs > 0 && mins > 0)
		fprintf(f, " %2d:%02d", hrs, mins);
	else if(hrs > 0)
		fprintf(f, " %d hour%s,", hrs, hrs>1?"s":"");
	else if(mins > 0)
		fprintf(f, " %d minute%s", mins, mins>1?"s":"");
	else
		fprintf(f, " %d second%s", secs, secs>1?"s":"");
	fprintf(f, "\n");

}

