/*-
 * Copyright (c) 2001, 2002 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: dump.c,v 1.9 2002/12/25 10:09:48 vlm Exp $
 */

#include "ipcad.h"
#include "cfgvar.h"
#include "opt.h"
#include "service.h"
#include "storage.h"

int
make_dump(char *dump_file, FILE *msgstream) {
	flow_t *flows, *flow;
	int entries, entr;
	time_t saved_time;
	time_t current_time;
	int df;
	struct stat sb;
	FILE *f;

	if(!dump_file || (!conf->chroot_to && *dump_file != '/')
	  || strstr(dump_file, "/..")
	  || !strncmp(dump_file, "/etc/", 5)
	  || !strncmp(dump_file, "/dev/", 5)
	) {
		fprintf(msgstream, "Incomplete (incorrect) pathname\n");
		return -1;
	};

	df = open(dump_file, O_CREAT | O_WRONLY
#ifdef	O_EXLOCK
		| O_EXLOCK
#endif
		, 0600);

	if(df == -1) {
		fprintf(msgstream, "Can't open dump file %s.\n", dump_file);
		return -1;
	};

	/* If file isn't locked by open(2) call, lock it manually. */
#ifndef	O_EXLOCK
	if( try_lock(df, 1) ) {
		fprintf(msgstream, "Can't lock dump file %s.\n", dump_file);
		close(df);
		return -1;
	}
#endif

	if(fstat(df, &sb) != 0) {
		fprintf(msgstream, "Can't fstat(%s).\n", dump_file);
		close(df);
		return -1;
	};

	if((sb.st_mode & S_IFMT) != S_IFREG) {
		fprintf(msgstream, "%s: Not a regular file.\n", dump_file);
		close(df);
		return -1;
	};

	if(sb.st_uid != getuid()) {
		fprintf(msgstream, "Invalid dump file owner.\n");
		close(df);
		return -1;
	};

	if((sb.st_mode & 0077)) {
		fprintf(msgstream, "Insecure dump file permissions.\n");
		close(df);
		return -1;
	};

	if( ftruncate(df, 0) != 0 ) {
		fprintf(msgstream, "Can't truncate file.\n");
		close(df);
		return -1;
	};

	if(!(f = fdopen(df, "w"))) {
		fprintf(msgstream, "Can't fdopen().\n");
		close(df);
		return -1;
	};

	lock_storage(&main_storage);
	{
		int ret;
		ret = get_flow_table(&main_storage, &flows, &entries);
		if(ret == -1) {
			fprintf(msgstream, "Memory allocation error\n");
			fprintf(msgstream, "Empty table saved to %s.\n",
				dump_file);
			fclose(f);
			unlock_storage(&main_storage);
			return 0;
		}

		saved_time = main_storage.create_time;
		time(&current_time);
	}
	unlock_storage(&main_storage);

	for(flow = flows, entr = 0; entr < entries; entr++, flow++) {
		print_aligned_ip(f, flow->src);
		print_aligned_ip(f, flow->dst);
		fprintf(f, "%18u %19qu\n",
			flow->packets, flow->bytes);
	}

	saved_time = (time(NULL) - saved_time) / 60;
	fprintf(f, "\nAccounting data age is %5ld\n", (long)saved_time);
	fprintf(f, "Accounting data saved %ld\n", (long)current_time);

	fprintf(msgstream, "%d elements saved to %s\n", entries, dump_file);
	fflush(f);
	fclose(f);

	if(flows) free(flows);

	return 0;
};


