/*-
 * Copyright (c) 2002 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: storage.c,v 1.3 2002/12/25 10:21:26 vlm Exp $
 */

#include "ipcad.h"
#include "cfgvar.h"
#include "storage.h"

/*
 * Functions prototypes.
 */
static void _aggregate_flow(flow_t *flow);
static void _storage_add(flow_storage_t *storage, flow_el_t *el);
static int _flow_hash_value(flow_t *flow);
static flow_t *_get_flow(flow_storage_t *storage, flow_t *key);

/*
 * Main storage and checkpoint storage.
 */
flow_storage_t main_storage;
flow_storage_t checkpoint_storage;


int
flow_update(flow_storage_t *storage, flow_t *flow) {
	flow_t *existing_flow;
	flow_el_t *el;

	_aggregate_flow(flow);

	lock_storage(storage);

	existing_flow = _get_flow(storage, flow);
	if(existing_flow) {
		existing_flow->packets += flow->packets;
		existing_flow->bytes += flow->bytes;
		unlock_storage(storage);
		return 0;
	}

	if(conf->memsize
	  && (storage->entries * sizeof(flow_el_t)) > conf->memsize) {

		if(storage->first_miss)
			time(&storage->first_miss);
		storage->missed_packets += flow->packets;
		storage->missed_bytes += flow->bytes;

		unlock_storage(storage);

		errno = ENOMEM;
		return -1;
	}

	el = malloc(sizeof *el);
	if(el == NULL) {
		if(storage->first_miss)
			time(&storage->first_miss);
		storage->missed_packets += flow->packets;
		storage->missed_bytes += flow->bytes;

		unlock_storage(storage);

		return -1;
	}

	memcpy(&el->flow, flow, sizeof(flow_t));

	_storage_add(storage, el);

	unlock_storage(storage);

	return -1;
}

void
clear_storage(flow_storage_t *storage) {
	flow_el_t *el, *next;

	lock_storage(storage);
		el = storage->head;
		memset(storage->buckets, 0,
			storage->numbuckets * sizeof(flow_el_t *));
		time(&storage->create_time);
		storage->head = 0;
		storage->entries = 0;
		storage->first_miss = 0;
		storage->missed_packets = 0;
		storage->missed_bytes = 0;
	unlock_storage(storage);

	/*
	 * Slowly and sadly delete all elements.
	 */
	for(; el; el = next) {
		next = el->hash_next;
		free(el);
	}
}

void
save_checkpoint(flow_storage_t *main, flow_storage_t *checkpoint) {
	flow_el_t **	cp_buckets;
	int		cp_numbuckets;

	lock_storage(main);

	clear_storage(checkpoint);

	lock_storage(checkpoint);
	assert(checkpoint->entries == 0);

	/*
	 * Save certain elements of the checkpoint storage.
	 */
	cp_buckets = checkpoint->buckets;
	cp_numbuckets = checkpoint->numbuckets;

	/*
	 * Move all the data from main to checkpoint.
	 */
#define	_move(foo) do { checkpoint->foo = main->foo; main->foo = 0; } while(0)
	_move(buckets);
	_move(head);
	_move(numbuckets);
	_move(entries);
	_move(create_time);
	_move(first_miss);
	_move(missed_packets);
	_move(missed_bytes);

	/*
	 * Restore certain elements from the checkpoint storage.
	 */
	main->buckets = cp_buckets;
	main->numbuckets = cp_numbuckets;

	unlock_storage(checkpoint);
	unlock_storage(main);
}


int
get_flow_table(flow_storage_t *storage, flow_t **r_flows, int *r_size) {
	int entries = storage->entries;
	flow_t *flows;
	flow_el_t *el;

	if(entries == 0) {
		*r_size = 0;
		*r_flows = NULL;
		return 0;
	}

	flows = malloc(entries * sizeof(flow_t));
	if(flows == NULL) {
		*r_size = -1;
		*r_flows = NULL;
		return -1;
	}

	*r_flows = flows;
	*r_size = entries;

	for(el = storage->head; el; el = el->hash_next) {
		*flows = el->flow;
		entries--;
		flows++;
	}

	assert(entries == 0);

	return 0;
}


/*
 * Internal functions.
 */

static void
_aggregate_flow(flow_t *flow) {
	unsigned long src = flow->src.s_addr;
	unsigned long dst = flow->dst.s_addr;
	struct atable *at;
	int did = 0;

	for(at = conf->atable; at; at = at->next) {
		if(!(did & 1) && ((src & at->mask.s_addr) == at->ip.s_addr)) {
			flow->src.s_addr = src & at->strip.s_addr;
			if((did |= 1) == 3)
				return;
		}

		if(!(did & 2) && ((dst & at->mask.s_addr) == at->ip.s_addr)) {
			flow->dst.s_addr = dst & at->strip.s_addr;
			if((did |= 2) == 3)
				return;
		}
	}

	return;
}

static void
_storage_add(flow_storage_t *storage, flow_el_t *el) {
	flow_el_t **bucket;

	if(storage->buckets == NULL) {
		int buckets = 65536 / sizeof(flow_el_t *);

		storage->buckets = calloc(buckets, sizeof(flow_el_t *));
		if(storage->buckets == NULL) {
			if(storage->first_miss)
				time(&storage->first_miss);
			storage->missed_packets += el->flow.packets;
			storage->missed_bytes += el->flow.bytes;
			free(el);
			return;
		}
		storage->numbuckets = buckets;
	}

	el->hash_value = _flow_hash_value(&el->flow);

	bucket = &storage->buckets[ el->hash_value % storage->numbuckets ];

	/*
	 * Add to the bucket (prepend).
	 */
	if(*bucket) {
		(*bucket)->bucket_prev = el;
		el->bucket_next = *bucket;
		el->bucket_prev = NULL;
		*bucket = el;
	} else {
		*bucket = el;
		el->bucket_next = NULL;
		el->bucket_prev = NULL;
	}

	/*
	 * Add to the list.
	 */
	el->hash_next = storage->head;
	storage->head = el;

	storage->entries++;
}

static int
_flow_hash_value(flow_t *flow) {
	int h;

	h = ntohl(flow->src.s_addr);
	h ^= ntohl(flow->dst.s_addr);

	return (h & 0x7fffffff);
}

static flow_t *
_get_flow(flow_storage_t *storage, flow_t *key) {
	int hash_value;
	flow_el_t **bucket, *el;
	struct in_addr src = key->src;
	struct in_addr dst = key->dst;

	if(storage->buckets == NULL)
		return NULL;

	hash_value = _flow_hash_value(key);

	bucket = &storage->buckets[ hash_value % storage->numbuckets ];
	for(el = *bucket; el; el = el->bucket_next) {
		if(el->flow.src.s_addr == src.s_addr
		&& el->flow.dst.s_addr == dst.s_addr) {
			if(el->bucket_prev) {
				/*
				 * Move this element to the top of bucket.
				 */
				if((el->bucket_prev->bucket_next
						= el->bucket_next))
					el->bucket_next->bucket_prev
						= el->bucket_prev;
				el->bucket_prev = NULL;
				el->bucket_next = *bucket;
				*bucket = el;
			}

			return &el->flow;
		}
	}

	return NULL;
}



