/*-
 * Copyright (c) 2002 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: storage.h,v 1.1 2002/12/25 10:09:48 vlm Exp $
 */

#ifndef	__STORAGE_H__
#define	__STORAGE_H__

#include "headers.h"

/*
 * Structure describing the single flow.
 */
typedef struct flow_s {
	/*
	 * Flow source and destination.
	 */
	struct in_addr src;
	struct in_addr dst;

	/*
	 * Flow counters.
	 */
	unsigned long long bytes;
	size_t packets;
} flow_t;

/*
 * Wrapper to include the flow into a hash.
 */
typedef struct flow_el_s {

	flow_t flow;	/* Flow itself */

	/*
	 * Hash value of the flow.
	 */
	int hash_value;

	/*
	 * Place in the bucket.
	 */
	struct flow_el_s *bucket_prev;
	struct flow_el_s *bucket_next;

	/*
	 * Place in the global list.
	 */
	struct flow_el_s *hash_next;
} flow_el_t;


/*
 * Hash of flows.
 */
typedef struct flow_storage_s {
	flow_el_t **buckets;
	flow_el_t *head;
	int numbuckets;
	int entries;

	time_t	create_time;

	time_t	first_miss;
	long long missed_packets;
	long long missed_bytes;

	pthread_mutex_t storage_lock;
} flow_storage_t;

/*
 * Main storage and checkpoint storage.
 */
extern flow_storage_t main_storage;
extern flow_storage_t checkpoint_storage;

#define	lock_storage(foo)	pthread_mutex_lock(&(foo)->storage_lock)
#define	unlock_storage(foo)	pthread_mutex_unlock(&(foo)->storage_lock)

/*
 * Atomically add flow counters into the list of flows.
 * flow will NOT be consumed (yet may be modified): you may allocate it
 * on the stack. The function may allocate new flow_el_t object or update
 * existing one.
 * If allocation fails, it will increase missed_flows in the storage
 * and return -1.
 */
int flow_update(flow_storage_t *storage, flow_t *flow);

/*
 * Clean flows from the storage.
 * Operation is atomic.
 */
void clear_storage(flow_storage_t *storage);

/*
 * Move main in place of checkpoint.
 * Operation is atomic.
 */
void save_checkpoint(flow_storage_t *main, flow_storage_t *checkpoint);

/*
 * Get a table with flows (array of flow_t).
 * If return value is 0, r_flows and r_size will be initialized
 * to point to a newly allocated table of flows.
 * You should lock storage before calling this function!
 */
int get_flow_table(flow_storage_t *storage, flow_t **r_flows, int *r_size);

#endif	/* __STORAGE_H__ */
